FUNCTION xsh_defaults,program
;+
; NAME:
;	XSH_DEFAULTS
;
; PURPOSE:
;	Define the data structure for each program of the Xop application.
;	The purpose of this function is to contain all the default input 
;	parameter definition in order to allow different Xop users to
;	customize their particular input defaults. 
;
; CATEGORY:
;	Xop application.
;
; CALLING SEQUENCE:
;
;	inp = XOP_DEFAULTS('xprogram')
;
; INPUTS:
;	xprogram:	Name of the program interface in Xop.
;
; OPTIONAL INPUTS:
;	
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;	The input structure with the data necessary to run the program.
;
; OPTIONAL OUTPUTS:
;
; COMMON BLOCKS:
;	shadow contains the default xshgeom and xshoe for faster
;	       access.
;
; SIDE EFFECTS:
;
; RESTRICTIONS: *** WARNING  ***
;	If the user changes this defaults, be careful with the type
;	of variable. Saying for instance a:4 initializes a as short
;	integer and WILL NEVER be a float. Place a:4.0 to set it as
;	float.
;
; PROCEDURE:
;	Straightforward.
;
; EXAMPLE:
;	inp = xop_defaults('xurgent')
;
; MODIFICATION HISTORY:
; 	Written by:	M. Sanchez del Rio
;	October, 1994	
;	95/10/24 MSR adds xopmap (mapping option)
;	95/10/31 MSR adds filedir keyword , to allow users to create
;		a file ~/.xop_defaults with his own customization.
;	96/11/05 MSR adds INITIALIZE
;       97/04/02 MSR adds ~/.xsh_defaults as defaults file.
;       98/01/06 MSR adds nrays in xsh_undul, converted to anonymous str.
;       98/03/19 MSR common block for faster access.
;       98/04/23 MSR adds default []range entry in plotxy and histo1. No
;		autorefresh by defaut.
;       98/04/30 MSR adds xsh_ray_prop
;	01/03/07 MSR fixes a bug in xshwig: ph1 and ph2 are now float.
;-

common shadow, xshgeom, xshoe, shadow_start

on_error,2
program = strupcase(program)

case program of
	'SHADOW_START': begin 
		case sdep() of
 		 'UNIX': BEGIN
		  if n_elements(shadow_start) EQ 0 then shadow_start = $
		   'source /scisoft/xop2.3/extensions/shadowvui/shadow-2.3.2m-linux/.shadowrc'
		  END
		 'WINDOWS':BEGIN
		  ;if n_elements(shadow_start) EQ 0 then shadow_start = $
		  shadow_start = $
		   ['set SHADOW_ROOT=c:/users/srio/shadow/SHADOW2.3.1',$
		   'set PATH=c:/users/srio/shadow/SHADOW2.3.1/bin;c:\windows\command.com',$
		     'SET SHADOW_ENV_FILE=xsh_env_file.inp']
		  END
		 else:
		endcase
		end
	'XSHUNDUL': begin 
		; when set flag to 0, then maps the corresponding program call
		xshundul  =  { $
			;epath
			lambdau : 0.0228,  $
			k : .256,  $
			e_energy : 6.04,  $
			;undul_set
			nperiods : 72,  $
			emin : 14705.0,  $
			emax : 14711.0,  $
			intensity : 0.2,  $
			maxangle : 0.015,  $
			;undul_phot
			undul_phot_flag : ['0','Shadow code','Urgent code'],$ 
			;input_source
			seed : 36255L, $
			sx : 345e-4, $
			sz : 23e-4, $
			ex : 4e-7, $
			ez : 4e-9, $
			nrays:5000L }
		end
	'XSHBM':begin
                 xshbm = { npoint:5000L, $
                          istar1:567656L, $
                          sigmax:7.8e-3, $
                          sigmaz:3.6e-3, $
                          hdiv1:0.0005, $
                          hdiv2:0.0005, $                         
                          r_aladdin_flag:['1','Magnetic Radius [m]',$
				'Magnetic Field [T]'],$
                          r_aladdin:0.8,$
                          epsi_x:3.8e-7,$
                          epsi_z:3.8e-9,$
                          bener:6.04,$
                          ph1:5000.0,$
                          ph2:100000.0}
                end
         'XSHWIG':begin
                     xshwig = { npoint:5000L, $
                              istar1:5676561L, $
                              sigmax:56e-4, $
                              sigmaz:5e-4, $
                              epsi_x:2.88e-7,$
                              epsi_z:2.88e-9,$
                              bener:2.01,$
                              ph1:10000.0,$
                              ph2:10010.0, $
                              periods:50, $
                              wavlen:0.04, $
                              K:7.85}
         
         
                end 	
         'XSHGEOM':begin
		if n_elements(xshgeom) EQ 0 then begin
		  file = 'start_geom.00'
		  dir = FilePath1(file)
		  IF dir EQ '' THEN BEGIN
		    itmp = Dialog_Message(Dialog_Parent=group,/Error,$
			'File '+file+' not found in directories in !path')
		    Return,0
		  ENDIF
		  xshgeom=read_gfile(dir+sdep(/ds)+file)
		endif
		end

         'XSH_SOURCE_G':begin
		;if n_elements(xsh_source_g) EQ 0 then begin
		  file = 'start_geom.00'
		  dir = FilePath1(file)
		  IF dir EQ '' THEN BEGIN
		    itmp = Dialog_Message(Dialog_Parent=group,/Error,$
			'File '+file+' not found in directories in !path')
		    Return,0
		  ENDIF
		  xsh_source_g=read_gfile(dir+sdep(/ds)+file)
		;endif
		end

         'XSH_SOURCE_BM':begin
		;if n_elements(xsh_source_bm) EQ 0 then begin
		  file = 'start_bm.00'
		  dir = FilePath1(file)
		  IF dir EQ '' THEN BEGIN
		    itmp = Dialog_Message(Dialog_Parent=group,/Error,$
			'File '+file+' not found in directories in !path')
		    Return,0
		  ENDIF
		  xsh_source_bm=read_gfile(dir+sdep(/ds)+file)
		;endif
		end

         'XSHOE':begin
		if n_elements(xshoe) EQ 0 then begin
		  file = 'start_oe.nn'
		  dir = FilePath1(file)
		  IF dir EQ '' THEN BEGIN
		    itmp = Dialog_Message(Dialog_Parent=group,/Error,$
			'File '+file+' not found in directories in !path')
		    Return,0
		  ENDIF
		  xshoe=read_gfile(dir+sdep(/ds)+file)
		endif
		end


	'XSH_PLOTXY': BEGIN
		xsh_plotxy = { $
		;
		;general
		;
		shadow_in:'begin.dat',$
		retrace:0.0,$
		col1:1, $
		col2:3, $
		XRANGE :[0.0,0.0], $
		YRANGE:[0.0,0.0], $
		NOLOST :0, $
		CART:0, $
		CONTOUR:0, $
		;
		; histograms
		;
		HISTO :0, $
		NBINS :25, $
		CALFWHM:0, $
		GAUSSFIT:0, $
		NC_BINS :25, $
		NLEVELS :6, $
		NSPLIT :1, $
		;
		; color
		;
		CFILE:'' , $
		CBINS:100, $
		CCOL:0, $
		CLIMITS:[0.0,0.0], $
		CREF:0, $
		;
		;presentation
		;
		TITLE :' ', $
		SLIT :[0.0,0.0,0.0,0.0], $
		NOCOMM :0, $
		CHARSIZE:0.0, $
		MULFAC :0.0, $
		PUBLISH :0, $
		SMOOTH :0, $
		; 
		; Other interface setups
		;
		AUTOREFRESH:0, $
		DEFAULT_XRANGE:1, $
		DEFAULT_YRANGE:1, $
		DEFAULT_CLIMITS:1 }
		;
		; not driven by the interface
		;
		;EXTRACT :0, INFOSH:0, NFILES :0, NOERASE :0, $
		;XYRANGE:[0.0,0.0], shadow_out:0 , FWHM_VAL :0, FWHM_FIT :0 }
		END
   'XSH_HISTO1':BEGIN		
		  xsh_histo1={ $
		  SHADOW_IN:'begin.dat', $
		  COL:6, $ 
		  NBINS:25,$		  	  	  
		  GAUSSFIT:0,$
		  XRANGE:[0.0,0.0],$
		  YRANGE:[0.0,0.0],$		  
		  YLOG:0,$
		  NOLOST:0,$
		  WRITE:0,$
		  CALFWHM:0,$
		  CALERROR:0,$
		  CM:0,$
		  REFLECTIVITY:0,$ ;  values=0,1,22,23,24,25,30,31,32,33
		  PUBLISH:0,$
		  TITLE:'',$
		  XTITLE:'',$
		  YTITLE:'',$
		  AUTOREFRESH:0, $
		  DEFAULT_XRANGE:1, $
		  DEFAULT_YRANGE:1, $
		  DEFAULT_CLIMITS:1 }
		END

   'XSH_RAY_PROP':BEGIN		
		  xsh_ray_prop={ $
		  SHADOW_IN:'star.01', $
		  COL1:1, $ 
		  COL2:3, $ 
		  RANGE:[-10.0,10.0],$
		  ITERS:21,$
		  XRANGE:[0.0,0.0],$
		  YRANGE:[0.0,0.0],$		  
		  TITLE:'',$
		  XTITLE:'',$
		  YTITLE:'',$
		  NOLOST:1,$
		  CART:0,$
		  XPLOT:0,$
		  MPEG:0,$
		  WAIT:0.0,$
		  DEFAULT_RANGE:0, $
		  DEFAULT_XRANGE:1, $
		  DEFAULT_YRANGE:1,$
                  NBINS:0L }
		END


	else:   begin
		print,'XSH_DEFAULTS: ERROR: Program not found'
		out = 0
		end
	endcase


;
; rename the structure to out 
;
out = 0
command = 'out = '+program
;print,command
tmp = execute(command)

;
; Overwrite data with values from defaults files
;
sep = sdep(/ds)
filedir = [Xop_GetEnv('SHADOWVUI_HOME')+sep+'data']
filedir = [filedir]
IF Xop_GetEnv('XOP_DEFAULTS_DIR') NE '' THEN $
   filedir = [filedir,Xop_GetEnv('XOP_DEFAULTS_DIR')]

nfiles = n_elements(filedir)
if nfiles EQ 1 then filedir = [filedir]
tmp=0
for i=0,nfiles-1 do begin
  file = filedir[i]+sep+StrLowCase(program)+'.xop'
  tmp = checkfile(file)
  if tmp EQ 1 then begin
    str_par = Xop_Input_Load(inputFile=file)
    ;IF N_Tags(out.parameters) NE 0 THEN BEGIN
    IF (Where(StrUpCase(Tag_Names(out)) EQ 'PARAMETERS'))(0) NE -1 THEN BEGIN
      tmp = out.parameters
      Copy_Structure,str_par, tmp
      out.parameters = tmp
    ENDIF ELSE BEGIN
      tmp = out
      Copy_Structure,str_par, tmp
      out = tmp
    ENDELSE
    message,/info,'Defaults from file: '+file+'
  endif
endfor
;
; return
;

return,out
end
